var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

import './polyfillFetchInEdge';
import 'isomorphic-fetch';

import createLogger from 'cf-util-logger';

var logRequest = createLogger('http:request');
var logError = createLogger('http:error');
var logSuccess = createLogger('http:success');

// Store beforeSend callbacks.
var beforeSendCallbacks = [];

/**
 * Modify request options before they are sent.
 * Useful for authentication or other middleware.
 *
 * ```js
 * beforeSend(function(opts) {
 *   opts.url = '/api/v4' + opts.url;
 * });
 * ```
 *
 * @param {Function} callback
 */
export function beforeSend(callback) {
  beforeSendCallbacks.push(callback);
}

export function clearBeforeSend() {
  beforeSendCallbacks = [];
}

function isJSON(contentType) {
  return (/[\/+]json\b/.test(contentType)
  );
}

function toQueryParams(kvs) {
  var queryParams = [];
  // Clones the input
  if (Array.isArray(kvs) || kvs instanceof Map) {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = kvs[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var _ref = _step.value;

        var _ref2 = _slicedToArray(_ref, 2);

        var k = _ref2[0];
        var v = _ref2[1];

        queryParams.push([k, v]);
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  } else {
    for (var _k in kvs) {
      if (kvs.hasOwnProperty(_k)) {
        queryParams.push([_k, kvs[_k]]);
      }
    }
  }
  return queryParams.map(function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 2),
        k = _ref4[0],
        v = _ref4[1];

    return encodeURIComponent(k) + '=' + encodeURIComponent(v);
  }).join('&');
}

function toHash(headers) {
  var hash = Object.create(null);
  if (headers._headers) {
    Object.keys(headers._headers).forEach(function (key) {
      var value = Array.isArray(headers._headers[key]) ? headers._headers[key][0] : headers._headers[key];
      hash[key] = value;
      hash[key.toLowerCase()] = value;
    });
  } else {
    // converting to Map allows for .. of in Edge
    var headersMap = new Map(headers);
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      for (var _iterator2 = headersMap[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        var _ref5 = _step2.value;

        var _ref6 = _slicedToArray(_ref5, 2);

        var k = _ref6[0];
        var v = _ref6[1];

        hash[k] = v;
        hash[k.toLowerCase()] = v;
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }
  }
  return hash;
}

function wrapResponse(headers, status, text, response) {
  var body = text;
  if (isJSON(headers['content-type'])) {
    body = text ? JSON.parse(text) : undefined;
  }
  return { headers: headers, status: status, body: body, text: text, response: response };
}

var BODYLESS_METHODS = ['GET', 'HEAD'];

/**
 * Perform an http request. This method uses WHATWG's fetch API and returns a
 * promise that **resolves on any HTTP status**, and rejects on anything else,
 * such as a malformed URL or a network error.

 * If a callback is provided, and `response.ok` is true, the node-style callback
 * will be called with an undefined error and a hash that contains the headers
 * key-value pairs, status code, deserializes body (JSON or plain-text only for
 * now), the body text, and the original `Response` object. Otherwise the
 * callback is called with a single argument containing the above hash as the
 * error value.
 *
 * This function also returns a promise that contains the fetched response
 * object.
 *
 * ```js
 * request('POST', '/posts', {
 *   body: {
 *     title: 'A New Post',
 *     content: 'Contents of the new post.'
 *   }
 * }).then(function(res) {
 *   res.json().then(console.log); // > { result: { id: 1, title: 'A New Post', content: 'Contents of the new post.' } }
 * });
 * ```
 *
 * @param {String} method - GET/POST/PUT/PATCH/DELETE
 * @param {String} url
 * @param {Object} [opts] Any options fetch()'s second parameter accepts
 * @param {Object|Array<Array>|Map} [opts.parameters] Query parameters to add to `url`
 * @returns {Promise} Fetch promise
 *
 * @see [Global Fetch on MDN](https://developer.mozilla.org/en-US/docs/Web/API/WindowOrWorkerGlobalScope/fetch)
 * @see [WHATWG Fetch API Spec](https://fetch.spec.whatwg.org/)
 */
export function request(method, url, opts, callback) {
  if (typeof opts === 'function') {
    callback = opts;
    opts = {};
  } else if (opts === null || typeof opts === 'undefined') {
    opts = {};
  }

  opts = Object.assign({}, opts);

  opts.method = method.toUpperCase();
  opts.url = url;
  opts.callback = callback;

  // Allow beforeSend to modify request options.
  beforeSendCallbacks.forEach(function (cb) {
    return cb(opts);
  });

  method = opts.method;
  url = opts.url;
  callback = opts.callback;

  // Fetch does not send cookies by default, this take fetch back to the
  // behavior similar to XHR
  if (!opts.credentials) opts.credentials = 'same-origin';

  // Normalize the headers
  opts.headers = Object.assign({}, opts.headers);

  // Emulate superagent's questionable ability to filter out headers that are
  // null or undefined.
  for (var h in opts.headers) {
    if (opts.headers[h] === null || typeof opts.headers[h] === 'undefined') {
      delete opts.headers[h];
    }
  }
  opts.headers = new Headers(opts.headers || {});

  // Emulate superagent's ability to automatically encode JSON body and set the
  // Content-Type
  if (!opts.skipBodyTransform && BODYLESS_METHODS.every(function (m) {
    return method.toUpperCase() !== m;
  }) && opts.body && _typeof(opts.body) === 'object') {
    opts.body = JSON.stringify(opts.body);
    if (!opts.headers.has('Content-Type')) {
      opts.headers.set('Content-Type', 'application/json');
    }
  }

  if (opts.parameters) {
    url += '' + (url.indexOf('?') === -1 ? '?' : '&') + toQueryParams(opts.parameters);
  }

  var logMessage = method + ' ' + url;
  logRequest(logMessage);

  return fetch(url, opts) // eslint-disable-line compat/compat
  .then(function (response) {
    var headers = toHash(response.headers);
    var status = response.status;
    logMessage = logMessage + ' (' + status + ' ' + response.statusText + ')';

    return response.text().then(function (text) {
      var wrappedResponse = wrapResponse(headers, status, text, response);
      if (response.ok) {
        logSuccess(logMessage);
        callback && callback(undefined, wrappedResponse);
        return wrappedResponse;
      }
      throw wrappedResponse;
    });
  }).catch(function (err) {
    logError(logMessage);
    callback && callback(err);
    throw err;
  });
}

/**
 * Perform a GET request.
 */
export function get() {
  for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  return request.apply(undefined, ['GET'].concat(args));
}

/**
 * Perform a POST request.
 */
export function post() {
  for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    args[_key2] = arguments[_key2];
  }

  return request.apply(undefined, ['POST'].concat(args));
}

/**
 * Perform a PUT request.
 */
export function put() {
  for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
    args[_key3] = arguments[_key3];
  }

  return request.apply(undefined, ['PUT'].concat(args));
}

/**
 * Perform a PATCH request.
 */
export function patch() {
  for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
    args[_key4] = arguments[_key4];
  }

  return request.apply(undefined, ['PATCH'].concat(args));
}

/**
 * Perform a DELETE request.
 */
export function del() {
  for (var _len5 = arguments.length, args = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
    args[_key5] = arguments[_key5];
  }

  return request.apply(undefined, ['DELETE'].concat(args));
}